/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import java.awt.Insets;
import javax.swing.*;
import jreqtools.gadgets.*;

/**
* A requester for simple strings.
* 
* @author	Markus Hillenbrand
*/

public class StringRequester extends AbstractRequester {
	private Object []		choices		= null;
	private Object 			selected	= null;
	private String 			string 		= "";
	private String			message	= null;
	private	StringChecker	checker		= null;
	private	int 				answer		= CANCEL;
	
	private class Dialog extends JDialog {
		private TextField textfield = new TextField(StringRequester.this.getString());
		
		public Dialog(JFrame parent)  {
			super(parent, "JReqTools: StringRequester", true);
			
			// Listener
			textfield.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } });
			textfield.addKeyListener(new KeyAdapter() { public void keyTyped(KeyEvent e) { if (e.getKeyChar() == KeyEvent.VK_ESCAPE) cancel(); } });
			textfield.setAlignmentX(CENTER_ALIGNMENT);
			textfield.setAlignmentY(CENTER_ALIGNMENT);
			
			// Buttons erzeugen
			OkCancelButtons buttons = new OkCancelButtons(StringRequester.this,
				new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } },
				new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); } },
				listener);
			
			Panel userpanel = new Panel();
			userpanel.setLayout(new BoxLayout(userpanel, BoxLayout.X_AXIS));
			userpanel.add(textfield);
			if (choices != null)
				{
				final JButton bt = new JButton(JReqTools.getIcon("choice"));
				bt.setAlignmentX(CENTER_ALIGNMENT);
				bt.setAlignmentY(CENTER_ALIGNMENT);
				bt.setMaximumSize(new Dimension(bt.getPreferredSize().width, bt.getPreferredSize().height));
				bt.setMargin(new Insets(1,1,1,1));
				bt.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { choice(bt); } });
				userpanel.add(bt);
				}
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(Box.createVerticalGlue());
			p.add(new MessagePanel(message, userpanel));
			p.add(Box.createVerticalGlue());
			p.add(new Strut());
			p.add(buttons);
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
		}

		private void cancel()  {
			answer = CANCEL;
			setVisible(false);
		}

		private void choice(JButton b)  {
			b.setEnabled(false);
			
			SelectRequester sr = new SelectRequester(
				JReqTools.getResourceString("DefaultChoice.Title"),
				JReqTools.getResourceString("DefaultChoice.Message"), 
				choices, selected);
				
			sr.setParentComponent(b);
			
			if (sr.request() == SelectRequester.OK)
				{
				textfield.setText( sr.getSelectedValue().toString() );
				}
			
			b.setEnabled(true);
		}

		private void ok()  {
			if (checker != null && ! checker.accept(textfield.getText()))
				{
				getToolkit().beep();
				JReqTools.showError(checker.getFailMessage());
				textfield.requestFocus();
				return;
				}
			
			answer = OK;
			setString(textfield.getText());
			setVisible(false);
		}

		public void setVisible(boolean b)  {
			if (b) textfield.requestFocus();
			super.setVisible(b);
		}

	}

	/**
	* Constructs a new StringRequester.
	*/

	public StringRequester()  {
		this("JReqTools: StringRequester", null, "");
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	defaultstring - The initial value of the text field.
	*/

	public StringRequester(String defaultstring)  {
		this("JReqTools: StringRequester", null, defaultstring);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public StringRequester(String message, String defaultstring)  {
		this("JReqTools: StringRequester", message, defaultstring);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public StringRequester(String title, String message, String defaultstring)  {
		setTitle(title);
		setMessage(message);
		setString(defaultstring);
	}

	protected javax.swing.JDialog createJDialog(JFrame parent)  {
		return new Dialog(parent);
	}

	protected int getReturnValue()  {
		return answer;
	}

	/**
	* Gets the string which has been entered by the user.
	*/

	public String getString()  {
		return string;
	}

	public void setDefaultChoices(Object [] choices)  {
		this.choices = choices;
		this.selected = null;
	}

	public void setDefaultChoices(Object [] choices, Object selected)  {
		this.choices = choices;
		this.selected = selected;
	}

	/**
	* Sets the message to be shown in the requester.
	*/

	public void setMessage(String newmessage)  {
		message = newmessage;
	}

	/**
	* Sets the string in the requester.
	*/

	public void setString(String newstring)  {
		string = newstring;
	}

	/**
	* Sets a StringChecker to be used for accepting an entered string.
	* 
	* @see	StringChecker
	* @see	StringCheckers
	*/

	public void setStringChecker(StringChecker newchecker)  {
		checker = newchecker;
	}

}
